@@ -5,9 +5,9 @@
 /* SYMPHONY was jointly developed by Ted Ralphs (ted@lehigh.edu) and         */
 /* Laci Ladanyi (ladanyi@us.ibm.com).                                        */
 /*                                                                           */
-/* (c) Copyright 2000-2010 Ted Ralphs. All Rights Reserved.                  */
+/* (c) Copyright 2000-2014 Ted Ralphs. All Rights Reserved.                  */
 /*                                                                           */
-/* This software is licensed under the Common Public License. Please see     */
+/* This software is licensed under the Eclipse Public License. Please see    */
 /* accompanying file for terms.                                              */
 /*                                                                           */
 /*===========================================================================*/
@@ -20,6 +20,9 @@
  * default. See below for the usage.
 \*===========================================================================*/
 
+#ifdef _OPENMP
+#include "omp.h"
+#endif
 
 #ifdef USE_OSI_INTERFACE
 
@@ -51,11 +54,14 @@ int main(int argc, char **argv)
 #include "symphony.h"
 #include "sym_master.h"
 #include "sym_messages.h"
-#ifdef HAS_READLINE
+#if defined HAS_READLINE
 #include <pwd.h>
 #include <readline/readline.h>
 #include <readline/history.h>
 
+#if (RL_VERSION_MAJOR == 5 && RL_VERSION_MINOR >= 2) || \
+   (RL_VERSION_MAJOR >= 6)
+
 typedef struct {
   char *name;			
 } COMMAND;
@@ -114,6 +120,10 @@ COMMAND parameter_commands[] = {
   { "gap_limit" },
   { "param_file" },
   { "do_primal_heuristic" },
+  { "should_use_rel_br" },
+  { "prep_level" },
+  { "tighten_root_bounds" },
+  { "limit_strong_branching_time" },
   { "back" },
   { "quit" },
   { "exit" },
@@ -125,6 +135,8 @@ void sym_initialize_readline();
 char *command_generator (const char *text, int state);
 char *alloc_str (char *s);
 void sym_read_tilde(char input[]);
+
+#endif
 #endif
 
 int comp_level = 0;
@@ -143,7 +155,6 @@ int main(int argc, char **argv)
 
    sym_environment *env = sym_open_environment();
    int termcode;
-
    
    if (argc > 1){
    
@@ -155,25 +166,29 @@ int main(int argc, char **argv)
       
       if (env->par.test){
 
-	 sym_test(env, &termcode);
+	 sym_test(env, argc, argv, &termcode);
 	 
       }else{
 	 
 	 if ((termcode = sym_load_problem(env)) < 0){
 	    printf("\nFatal errors encountered. Exiting with code %i.\n",
 		   termcode);
-	    printf("See sym_return_values.h for meaning of code.\n\n");
+	    printf("See symphony.h for meaning of code.\n\n");
 	    exit(termcode);
 	 }
 	 
 	 if ((termcode = sym_find_initial_bounds(env)) < 0){
 	    printf("\nFatal errors encountered. Exiting with code %i.\n",
 		   termcode);
-	    printf("See sym_return_values.h for meaning of code.\n\n");
+	    printf("See symphony.h for meaning of code.\n\n");
 	    exit(termcode);
 	 }
 	 printf("\n");
-	 sym_solve(env);
+	 if (env->mip->obj2 != NULL){
+	    sym_mc_solve(env);
+	 } else {
+	    sym_solve(env);
+	 }
       }
    
    } else{
@@ -190,11 +205,14 @@ int main(int argc, char **argv)
      double finish_time = 0.0, dbl_value = 0;
 
      sym_version();
-     printf("Please type 'help'/'?' to see a list of commands\n\n");
+     printf("***** WELCOME TO SYMPHONY INTERACTIVE MIP SOLVER ******\n\n"
+	    "Please type 'help'/'?' to see the main commands!\n\n");
 
      sym_set_int_param(env, "verbosity", -1);
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+ 			      (RL_VERSION_MAJOR >= 6))
      sym_initialize_readline();
 #endif
      
@@ -218,7 +236,9 @@ int main(int argc, char **argv)
 	   strcpy(args[1], line);
 	 }	 
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
 	 sym_read_tilde(args[1]);	 
 #endif	 	 
 	 if (fopen(args[1], "r") == NULL){
@@ -275,7 +295,9 @@ int main(int argc, char **argv)
 	     strcpy(args[2], line);
 	   }
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
 	   sym_read_tilde(args[2]);	 
 #endif	 	 
 	 
@@ -299,14 +321,22 @@ int main(int argc, char **argv)
 	 if(strcmp(args[0], "solve") == 0){
 	   start_time = wall_clock(NULL);
 	   printf("\n");
-	   termcode = sym_solve(env);
+	   if (env->mip->obj2 != NULL){
+	      termcode = sym_mc_solve(env);
+	   } else {
+	      termcode = sym_solve(env);
+	   }
 	   finish_time = wall_clock(NULL);
 	 } else {
 	   is_int = env->mip->is_int;
 	   env->mip->is_int  = (char *)   calloc(CSIZE, env->mip->n);
 	   start_time = wall_clock(NULL);
 	   printf("\n");
-	   termcode = sym_solve(env);
+	   if (env->mip->obj2 != NULL){
+	      termcode = sym_mc_solve(env);
+	   } else {
+	      termcode = sym_solve(env);
+	   }
 	   finish_time = wall_clock(NULL);
 	   env->mip->is_int = is_int;
 	   is_int = 0;
@@ -385,7 +415,7 @@ int main(int argc, char **argv)
 			 printf("Nonzero column names and values in the solution\n");
 			 printf("+++++++++++++++++++++++++++++++++++++++++++++++\n");
 			 for(j = 0; j<env->best_sol.xlength; j++){		      
-			    printf("%8s %10.3f\n", 
+			    printf("%8s %10.10f\n", 
 				   env->mip->colname[env->best_sol.xind[j]],
 				   env->best_sol.xval[j]);
 			 }
@@ -395,7 +425,7 @@ int main(int argc, char **argv)
 			 printf("User indices and values in the solution\n");
 			 printf("+++++++++++++++++++++++++++++++++++++++++++++++\n");
 			 for(j = 0; j<env->best_sol.xlength; j++){		      
-			    printf("%7d %10.3f\n", env->best_sol.xind[j], 
+			    printf("%7d %10.10f\n", env->best_sol.xind[j], 
 				   env->best_sol.xval[j]);
 			 }			    
 			 printf("\n");
@@ -429,7 +459,7 @@ int main(int argc, char **argv)
 				env->warm_start->ub, env->warm_start->lb, 
 				initial_time, start_time, finish_time,
 				env->mip->obj_offset, env->mip->obj_sense,
-				env->warm_start->has_ub,NULL);
+				env->warm_start->has_ub,NULL, 0);
 	       printf("\n");	       
 	     }
 	     strcpy(args[1], "");	       
@@ -514,7 +544,9 @@ int main(int argc, char **argv)
 	       strcpy(args[2], line);
 	     }
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) ||\
+			      (RL_VERSION_MAJOR >= 6))
 	     sym_read_tilde(args[2]);	 
 #endif	 	 
 
@@ -639,6 +671,10 @@ int sym_help(const char *line)
 	   "gap_limit                          : set the target gap between the lower and upper bound\n"
            "param_file                         : read parameters from a parameter file\n"
 	   "do_primal_heuristic                : whether or not to use primal heuristics\n\n"
+	   "should_use_rel_br                  : whether or not to use reliability branching\n\n"
+	   "prep_level                         : pre-processing level\n\n"
+	   "tighten_root_bounds                : whether to tighten root bounds \n\n"
+	   "limit_strong_branching_time        : whether to limit time spent in strong branching \n\n"
 	   "back                               : leave this menu\n"
 	   "quit/exit                          : leave the optimizer\n\n");
 					    
@@ -663,7 +699,25 @@ int sym_help(const char *line)
 int sym_read_line(const char *prompt, char **input)
 {
 
-#ifndef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
+
+  if (*input) FREE(*input);
+
+  while(true) {
+    *input = readline(prompt);
+    if (**input) {      
+      add_history(*input);
+      if((*input)[strlen(*input)-1] == ' '){
+	(*input)[strlen(*input)-1] = 0;
+      }
+      break;
+    } else continue;
+  }
+
+#else
+  
   int i;
 
   if (*input) FREE(*input);
@@ -671,7 +725,7 @@ int sym_read_line(const char *prompt, char **input)
    
   while(true){
      strcpy(getl, "");
-     printf(prompt);
+     printf("%s",prompt);
      fflush(stdout);
      fgets(getl, MAX_LINE_LENGTH, stdin);
 
@@ -689,23 +743,8 @@ int sym_read_line(const char *prompt, char **input)
   }
   *input = getl;
   
-#else
-
-  if (*input) FREE(*input);
-
-  while(true) {
-    *input = readline(prompt);
-    if (**input) {      
-      add_history(*input);
-      if((*input)[strlen(*input)-1] == ' '){
-	(*input)[strlen(*input)-1] = 0;
-      }
-      break;
-    } else continue;
-  }
-
 #endif
-  
+
   return (0);
 }
 
@@ -726,7 +765,9 @@ int sym_free_env(sym_environment *env){
 } 
 /*===========================================================================*\
 \*===========================================================================*/
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
 
 void sym_initialize_readline()
 {
